@@ -196,7 +196,11 @@ class ConfigEndpoint(RestEndpoint):
                 )
 
             if core_config and not (
-                "global_brightness" in core_config and len(core_config) == 1
+                any(
+                    key in core_config
+                    for key in ["global_brightness", "create_segments"]
+                )
+                and len(core_config) == 1
             ):
                 self._ledfx.loop.call_soon_threadsafe(self._ledfx.stop, 4)
 
