@@ -1,9 +1,10 @@
 import logging
+from json import JSONDecodeError
 
 from aiohttp import web
 
 from ledfx.api import RestEndpoint
-from ledfx.utils import async_fire_and_forget
+from ledfx.utils import async_fire_and_forget, set_name_to_icon
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -13,8 +14,18 @@ class FindDevicesEndpoint(RestEndpoint):
 
     ENDPOINT_PATH = "/api/find_devices"
 
-    async def post(self) -> web.Response:
+    async def post(self, request) -> web.Response:
         """Find and add all WLED devices on the LAN"""
+        try:
+            data = await request.json()
+        except JSONDecodeError:
+            response = {
+                "status": "failed",
+                "reason": "JSON Decoding failed",
+            }
+            return web.json_response(data=response, status=400)
+
+        set_name_to_icon(data.get("name_to_icon"))
 
         def handle_exception(future):
             # Ignore exceptions, these will be raised when a device is found that already exists
