@@ -10,7 +10,9 @@ import re
 import socket
 import sys
 import time
+import timeit
 from abc import ABC
+from collections import deque
 from collections.abc import MutableMapping
 from functools import lru_cache
 from itertools import chain
@@ -26,6 +28,21 @@ from ledfx.config import save_config
 
 # from asyncio import coroutines, ensure_future
 
+try:
+    from itertools import cycle
+
+    from bokeh.io import output_file, show
+    from bokeh.layouts import column
+    from bokeh.models import Label
+    from bokeh.palettes import Category10
+    from bokeh.plotting import figure
+
+    from ledfx.config import get_default_config_directory
+
+    bokeh_available = True
+except ImportError:
+    bokeh_available = False
+
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -167,7 +184,7 @@ def async_fire_and_return(coro, callback, timeout=10):
         exc = future.exception()
         if exc:
             # Handle wonderful empty TimeoutError exception
-            if type(exc) == TimeoutError:
+            if isinstance(exc, TimeoutError):
                 _LOGGER.warning(f"Coroutine {future} timed out.")
             else:
                 _LOGGER.error(exc)
@@ -311,7 +328,8 @@ class WLED:
         Returns:
             dict: array of segments
         """
-        return await self.get_state()["seg"]
+        res = await self.get_state()
+        return res["seg"]
 
     async def set_power_state(self, state):
         """
@@ -530,7 +548,7 @@ async def resolve_destination(
 
 
 def currently_frozen():
-    """Checks to see if running in a frozen environment such as pyinstaller or pyupdater package
+    """Checks to see if running in a frozen environment such as pyinstaller created binaries
     Args:
         Nil
 
@@ -926,3 +944,260 @@ class RegistryLoader:
 
     def get(self, *args):
         return self._objects.get(*args)
+
+
+class Plot_range:
+    def __init__(self, key, birth, points=1000):
+        self.key = key
+        self.xs = deque(maxlen=points)
+        self.ys = deque(maxlen=points)
+        self.birth = birth
+
+    def append(self, y):
+        self.xs.append(timeit.default_timer() - self.birth)
+        self.ys.append(y)
+
+    def list_x(self):
+        return list(self.xs)
+
+    def list_y(self):
+        return list(self.ys)
+
+
+class Tag:
+    def __init__(self, x, y, text, color="black"):
+        self.x = x
+        self.y = y
+        self.text = text
+        self.color = color
+
+
+class Graph:
+    """
+    Graph is a simple wrapper for bokeh to give high value multi range
+    time domain graphs with absolute minimum code
+    Supports mulitple ranges, and text tags
+
+    Lifecycle:
+        myGraph=("Animal hunt", ["Frogs", "Elephants"], y_title="Distance")
+        ...
+        myGraph.append_by_key("Frogs", 2.7)
+        myGraph.append_by_key("Elephants", 9.2)
+        ...
+        myGraph.append_by_key("Elephants", 6.0)
+        myGraph.append_tag("I am hungry", 1.0, color="red")
+
+        myGraph.dump_graph()
+    """
+
+    def __init__(
+        self,
+        title,
+        keys,
+        points=1000,
+        tags=10,
+        y_title="plumbus",
+        y_axis_max=None,
+    ):
+        """
+        Creates a graph instance, sets X axis to 0 seconds
+
+        Parameters:
+            title (str): String title to be displayed on graph
+            keys (list[(str)]: list of range titles to be display in key and available to append data values to
+            points (int): how many points to support in rolling buffer
+            tags (int): how many text tags to support in rolling buffer
+            y_title (str): Axis title for Y range
+            y_axis_max (float): If not None, will force the y axis max
+        """
+        self.title = title
+        self.y_title = y_title
+        self.y_axis_max = y_axis_max
+        self.ranges = {}
+        self.keys = keys
+        self.birth = timeit.default_timer()
+        for key in keys:
+            self.ranges[key] = Plot_range(key, self.birth, points=points)
+        self.tags = deque(maxlen=tags)
+
+    def append_by_key(self, key, value):
+        """
+        Appends a value into range ring buffer associated with axis key, timestamp is applied in second since graph creation
+
+        Parameters:
+            key (str): key name of the range, matching those used during creation to which to append
+            value (float): value which you wish to append to the range
+        """
+        self.ranges[key].append(value)
+
+    def append_tag(self, text, y, color="black"):
+        """
+        Appends a text tag into tag ring buffer, timestamp is applied in seconds since graph creation
+
+        Parameters:
+            text (str): text to be displayed as tag
+            y (float): value which you wish to display the tag
+        """
+
+        self.tags.append(
+            Tag(timeit.default_timer() - self.birth, y, text, color=color)
+        )
+
+    def dump_graph(self, sub_title=None, jitter=False, only_jitter=False):
+        """
+        Will spawn an interaction graph session into the browser
+
+        Parameters:
+            sub_title (str): Optional sub title to add to the base title
+                             Useful for when you want to know why the graph
+                             was dumped
+            jitter (bool): If true, will dump the jitter graph
+            only_jitter (bool): If true, will only dump the jitter graph
+        """
+        if not bokeh_available:
+            _LOGGER.info("Bokeh is disabled dump is disabled")
+        else:
+            if sub_title:
+                compound = f"{self.title} : {sub_title}"
+            else:
+                compound = self.title
+
+            _LOGGER.info(f"Attempting to dump graph {compound}")
+            TOOLS = "xpan,xwheel_zoom,box_zoom,reset,save,box_select"
+            colors = cycle(Category10[10])
+
+            vals_fig = figure(
+                title=compound,
+                x_axis_label="sec since start",
+                y_axis_label=self.y_title,
+                tools=TOOLS,
+                active_scroll="xwheel_zoom",
+                width=1200,
+                height=600,
+            )
+
+            for a_range in self.ranges.values():
+                if len(a_range.list_x()) > 0:
+                    vals_fig.line(
+                        a_range.list_x(),
+                        a_range.list_y(),
+                        legend_label=a_range.key,
+                        line_width=2,
+                        color=next(colors),
+                    )
+
+            for tag in self.tags:
+                label = Label(
+                    x=tag.x,
+                    y=tag.y,
+                    text=tag.text,
+                    text_font_size="12pt",
+                    text_color=tag.color,
+                    angle=1.57,
+                )
+                vals_fig.add_layout(label)
+
+            if self.y_axis_max is not None:
+                vals_fig.y_range.end = self.y_axis_max
+
+            vals_fig.legend.click_policy = "hide"
+
+            if jitter or only_jitter:
+                jitter_title = f"{compound} jitter"
+
+                jitter_fig = figure(
+                    title=jitter_title,
+                    x_axis_label="sec since start",
+                    x_range=vals_fig.x_range,
+                    y_axis_label="periodic secs",
+                    tools=TOOLS,
+                    active_scroll="xwheel_zoom",
+                    width=1200,
+                    height=600,
+                )
+
+                for a_range in self.ranges.values():
+                    # Calculte jitter for range x and prestuff so len is same
+                    # don't use numpy due to some side effects
+                    x = a_range.list_x()
+                    if len(x) > 0:
+                        jitter = [x[i + 1] - x[i] for i in range(len(x) - 1)]
+                        jitter.insert(0, 0.0)
+                        jitter_fig.circle(
+                            a_range.list_x(),
+                            jitter,
+                            legend_label=a_range.key,
+                            size=3,
+                            color=next(colors),
+                        )
+
+                for tag in self.tags:
+                    label = Label(
+                        x=tag.x,
+                        y=0.001,
+                        text=tag.text,
+                        text_font_size="12pt",
+                        text_color=tag.color,
+                        angle=1.57,
+                        text_baseline="middle",
+                    )
+
+                    jitter_fig.add_layout(label)
+
+                jitter_fig.legend.click_policy = "hide"
+
+            # work out layour according to requested graphs
+            if only_jitter:
+                p = column(jitter_fig)
+            elif jitter:
+                p = column(vals_fig, jitter_fig)
+            else:
+                p = column(vals_fig)
+
+            save_as = os.path.join(
+                get_default_config_directory(),
+                f"{re.sub('[^A-Za-z0-9]+', '_', compound)}.html",
+            )
+            output_file(filename=save_as, title=compound)
+            show(p)
+
+
+def wled_support_DDP(build) -> bool:
+    # https://github.com/Aircoookie/WLED/blob/main/CHANGELOG.md#build-2110060
+    if build >= 2110060:
+        return True
+    else:
+        return False
+
+
+def clean_ip(ip_address):
+    """Strip common error input from IP copy from chrome that is actually a URL
+
+    Args:
+        ip_address (string): The IP address to be cleaned
+
+    Returns:
+        string: The cleaned IP address
+    """
+
+    return (
+        ip_address.replace("https://", "")
+        .replace("http://", "")
+        .replace("/", "")
+    )
+
+
+name_to_icon = {}
+
+
+def set_name_to_icon(new_dict):
+    global name_to_icon
+    name_to_icon = new_dict
+
+
+def get_icon_name(wled_name):
+    global name_to_icon
+    for name, icon in name_to_icon.items():
+        if name.lower() in wled_name.lower():
+            return icon
+    return "wled"
